﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using DeukBot4.APIHandlers;
using DeukBot4.Utilities;
using Discord;
using Discord.WebSocket;

namespace DeukBot4.MessageHandlers
{
    public static class MainHandler
    {
        public static async Task HandleMessage(SocketMessage message)
        {
            if (message.Author.Id == Program.Client.CurrentUser.Id)
            {
                return;
            }
            try
            {
#pragma warning disable 4014
                CommandHandler.CommandHandler.HandleMessage(message);
                HandlePrivateMessage(message);
                ImageBackupHandler.Backup(message);
                DeltaHandler(message);
                DadJokeHandler(message);
#pragma warning restore 4014
            }
            catch (Exception e)
            {
                await Logger.LogError(e.ToString());
            }
        }

        private static ITextChannel _dmChannel;
        private static async Task HandlePrivateMessage(SocketMessage message)
        {
            if (message.Channel is IPrivateChannel)
            {
                await Logger.Log(($"Private Message: {message.Author.Username}- {message.Content}"));
                if (_dmChannel == null)
                {
                    _dmChannel = (ITextChannel) Program.Client.GetChannel(Program.Settings.DmChannel);
                }
                var eb = new EmbedBuilder
                {
                    Author = new EmbedAuthorBuilder
                    {
                        Name = message.Author.Username,
                        IconUrl = message.Author.GetAvatarUrl(),
                    },
                    Description = message.Content,
                    Color       = Color.Gold,
                    Timestamp = message.CreatedAt,
                    Footer = new EmbedFooterBuilder
                    {
                        Text = $"User ID: {message.Author.Id}"
                    }
                };
                await _dmChannel.SendMessageAsync("", embed: eb.Build());
            }
        }

        private static async Task DeltaHandler(SocketMessage message)
        {
            var lowerCasedContent = message.Content.ToLowerInvariant().RemoveSpecialCharacters();
            if (lowerCasedContent.Contains("origin"))
                return;

            var lowerSplit = lowerCasedContent.Split(' ');
            if (lowerSplit.Select(s => Lehvenstein.LevenshteinDistance(s, "delta")).Any(diff => diff <= 1))
            {
                var warning = "uhh excuse me it's called\n Origin and it's an art";
                await message.Channel.SendFileAsync(await CatPicHandler.GetCatPicture(warning), "cat_pic.png");
            }

        }

        private static async Task DadJokeHandler(SocketMessage message)
        {
            if (!(message.Author is IGuildUser guildUser))
                return;
            string newName;
            var content = message.Content;
            var lower = content.ToLowerInvariant();

            if (lower.StartsWith("hi "))
            {
                lower = lower.Remove(0, 3);
                content = content.Remove(0, 3);
            }
            else if (lower.StartsWith("hello "))
            {
                lower   = lower.Remove(0, 6);
                content = content.Remove(0, 6);
            }
            if (lower[0] != 'i')
                return;
            if (lower.StartsWith("i'm "))
            {
                newName = content.Remove(0, 4);
            }
            else if (lower.StartsWith("im "))
            {
                newName = content.Remove(0, 3);
            }
            else if (lower.StartsWith("i am "))
            {
                newName = content.Remove(0, 5);
            }
            else
            {
                return;
            }
            if (newName.Length > 15)
                return;

            await message.Channel.SendMessageAsync($"Hi {newName}, i'm Deukbot");
            await guildUser.ModifyAsync(user => user.Nickname = newName);

        }
    }
}